from .lib.openai_assistant import OpenAIAssistant

class Reviewer:
    def __init__(self, openai_api_key, assistant_id, mode='production'):
        self.assistant = OpenAIAssistant(openai_api_key, assistant_id, mode=mode)
        self.steps = list()

    def add_item(self, item):
        response = self.assistant.add_and_retrieve_message(item, "user")
        if response and response != 'APIError: No response':
            return response
        return None

    def append_item(self, item=""):
        self.__append_step(self.add_item, item)

    def execute(self, callback):
        for step in self.steps:
            callback(step())

    def __append_step(self, func, *args, **kwargs):
        def wrapper(*args2, **kwargs2):
            # print(f"Running step: {func.__name__}")
            return func(*args, **kwargs)

        self.steps.append(wrapper)




from typing import Any
from .env_modes import EnvModes

dummy_content = {'content': [{'text': {'value': 'Hello, world!'}}]}

class AIEnv(EnvModes):

    def __init__(self, mode='production', *args, **kwargs) -> None:
        super().__init__(mode)

    @EnvModes.non_production_handler({'id': 'non_prod', **dummy_content})
    def add_message(self, message, role):
        return "Funciona para todo ambiente"

    @EnvModes.non_production_handler({'data': [dummy_content]})
    def get_messages_list(self):
        return []
    
    @EnvModes.non_production_handler(dummy_content)
    def get_last_message(self):
        return {}

    @EnvModes.non_production_handler({'id': 'non_prod'})   
    def create_thread_run(self):
        return {'id': 'prod'}

    @EnvModes.non_production_handler({'id': 'non_prod', 'status': 'completed'})
    def wait_on_run(self, run):
        return {'id': 'non_prod', 'status': 'completed'}

    @EnvModes.non_production_handler('Hello, world!')
    def add_and_retrieve_message(self, message, role):
        return 'Hello, world!'
    


if __name__ == '__main__':
    ai_env_dev = AIEnv(mode='development')
    print(ai_env_dev.add_message('Hello, world!', 'user'))
    ai_env = AIEnv(mode='production')
    print(ai_env.add_message('Hello, world!', 'user'))



import unittest
from cururo.lib.env_modes import EnvModes 

class TestEnvModes(unittest.TestCase):

    def setUp(self):
        self.env_modes = EnvModes(mode='development')

    def test_instance(self):
        self.assertIsInstance(self.env_modes, EnvModes)

    def test_valid_mode(self):
        self.assertEqual(self.env_modes.mode, 'development')
        self.env_modes('testing')
        self.assertEqual(self.env_modes.mode, 'testing')

    def test_invalid_mode(self):
        with self.assertRaises(AssertionError):
            self.env_modes('invalid_mode')

    def test_str_representation(self):
        self.assertEqual(str(self.env_modes), "EnvModes(mode=development)")

    def test_repr_representation(self):
        self.assertEqual(repr(self.env_modes), "EnvModes(mode=development)")

    # def test_non_production_handler(self):
    #     pass # to complete

if __name__ == '__main__':
    unittest.main()



import time
from openai import OpenAI
from .ai_env import AIEnv, EnvModes, dummy_content
from ..util.util import handle_errors

class OpenAIAssistant(AIEnv):
    def __init__(self, api_key, assistant_id, thread_id=None, mode='production'):
        super().__init__(mode)
        self.client = OpenAI(api_key=api_key)
        self.assistant_id = assistant_id

        self.thread_id = thread_id if thread_id else self.__create_thread()

    @EnvModes.non_production_handler('non_prod')
    @handle_errors(default_value='Failed to create thread')
    def __create_thread(self):
        response = self.client.beta.threads.create()
        return response.id
    
    @EnvModes.non_production_handler({'id': 'non_prod', **dummy_content})
    @handle_errors(default_value='Failed to add message')
    def add_message(self, message, role):
        return self.client.beta.threads.messages.create(
            thread_id=self.thread_id,
            role=role,
            content=message
        )
    
    @EnvModes.non_production_handler({'data': [dummy_content]})
    @handle_errors(default_value='Failed to get messages list')
    def get_messages_list(self):
        return self.client.beta.threads.messages.list(thread_id=self.thread_id)
    
    @EnvModes.non_production_handler(dummy_content)
    @handle_errors(default_value='Failed to get last message')
    def get_last_message(self):
        return self.get_messages_list().data[0]
    
    @EnvModes.non_production_handler({'id': 'non_prod'})
    @handle_errors(default_value='Failed to create thread run')
    def create_thread_run(self):
        return self.client.beta.threads.runs.create(
            thread_id=self.thread_id,
            assistant_id=self.assistant_id,
        )
    
    @EnvModes.non_production_handler({'id': 'non_prod', 'status': 'completed'})
    @handle_errors(default_value='Failed to wait on run')
    def wait_on_run(self, run):
        while run.status == "queued" or run.status == "in_progress":
            run = self.client.beta.threads.runs.retrieve(
                thread_id=self.thread_id,
                run_id=run.id,
            )
            time.sleep(0.5)
        return run
    
    @EnvModes.non_production_handler('Hello, world!')
    @handle_errors(default_value='APIError: no response')
    def add_and_retrieve_message(self, message, role):
        message = self.add_message(message, role)
        run = self.create_thread_run()
        status = self.wait_on_run(run)
        msg = self.get_last_message()
        return msg.content[0].text.value
    
if __name__ == '__main__':
    assistant = OpenAIAssistant('a', 'b', mode='development')
    print(assistant.add_and_retrieve_message('Hello, world!', 'user'))


import inspect
import json

class Publisher:
    def __init__(self, *args, **kwargs):
        pass

    def publish(self, response):
        if response is None:  # Check if there is a response to publish
            return
        
        # Dynamically find and run all methods starting with __create_
        methods = inspect.getmembers(self, predicate=inspect.ismethod)
        for name, method in methods:
            mangled_name = f"_{self.__class__.__name__}__create_"
            if name.startswith(mangled_name):
                method(response)

    @staticmethod
    def parse_response(response_string):
        """
        Parse the JSON string into a Python dictionary.
        """
        try:
            return json.loads(response_string)
        except json.JSONDecodeError:
            return {'message': response_string}



import unittest
from .linter_test import LinterTest
from cururo.util.publisher import Publisher

class TestPublisher(LinterTest):

    def setUp(self):
        self.publisher = Publisher()

    def test_publisher(self):
        self.assertIsInstance(self.publisher, Publisher)
    
    def test_has_publish(self):
        self.assertTrue(hasattr(self.publisher, 'publish'))

    def test_publish(self):
        self.assertTrue(hasattr(self.publisher.publish, '__call__'))

if __name__ == '__main__':
    unittest.main()


class ResponseHandler:
    def __init__(self, publisher=None, logger=None, additional_actions=None):
        """
        Initializes the response handler with optional publisher, logger, and additional actions.

        Args:
        publisher (Publisher): An instance of Publisher class for publishing results.
        logger (Logger): A logging instance to log responses.
        additional_actions (list of callable): Additional actions to be executed with the response.
        """
        self.publisher = publisher
        self.logger = logger
        self.additional_actions = additional_actions if additional_actions else []

    def handle_response(self, response):
        """
        Handles the response using configured methods.

        Args:
        response (str): The response to handle.
        """
        if self.publisher and response:
            self.publisher.publish(response)

        if self.logger:
            self.logger.log(response)

        for action in self.additional_actions:
            action(response)



import argparse
import os
from .reviewer import Reviewer
from .lib.web_publisher import WebPublisher
from .util.handler import ResponseHandler
from .lib.env_modes import EnvModes

def custom_action(response):
    print(f"CURURO_RESPONSE([C:START]{response}[C:END])")

DEFAULT_OPENAI_KEY = os.getenv('OPENAI_KEY')
DEFAULT_ASSISTANT_ID = os.getenv('ASSISTANT_ID')
DEFAULT_MODE = 'production'
DEFAULT_PUBLISHER = None


def main():
    parser = argparse.ArgumentParser(description='Review an item using OpenAI.')
    parser.add_argument('--item', type=str, required=True, help='Item to add to the review.')
    parser.add_argument('--openai-key', type=str, default=DEFAULT_OPENAI_KEY, help='OpenAI API key.', required=DEFAULT_OPENAI_KEY is None)
    parser.add_argument('--assistant-id', type=str, default=DEFAULT_ASSISTANT_ID, help='OpenAI assistant ID.', required=DEFAULT_ASSISTANT_ID is None)
    parser.add_argument('--mode', type=str, default=DEFAULT_MODE, help='Mode to run the review in.', choices=EnvModes.allowed_modes)
    parser.add_argument('--web-url', type=str, help='URL to post the response to.', default=None)
    parser.add_argument('--web-secret', type=str, help='Secret to use when posting the response to the URL.', default=None)
    parser.add_argument('--web-processor', type=str, help='Processor to use when posting the response to the URL.', default=None)
    parser.add_argument('--version', action='version', version='%(prog)s 1.1.3')
    
    args = parser.parse_args()

    if args.web_url and not args.web_secret:
        parser.error('--web-secret is required when using --web-url')

    if args.web_processor:
        try:
            processor = eval(args.web_processor)
        except:
            parser.error('--web-processor must be a valid callable')
        args.web_processor = processor

    # check url
    if args.web_url:
        publisher = WebPublisher(web_url=args.web_url, web_secret=args.web_secret, processor=args.web_processor)
    else:
        publisher = DEFAULT_PUBLISHER

    handler = ResponseHandler(publisher=publisher, additional_actions=[custom_action])
    reviewer = Reviewer(openai_api_key=args.openai_key, assistant_id=args.assistant_id, mode=args.mode)
    reviewer.append_item(args.item)
    reviewer.execute(handler.handle_response)

if __name__ == '__main__':
    main()



import unittest


class LinterTest(unittest.TestCase):

    def asssertWarning(self, warningsA, warningsB):
        self.assertEqual(len(warningsA), len(warningsB), 'The number of warnings do not match')
        for x, y in zip(warningsA, warningsB):
            self.assertEqual(x.__str__(), y.__str__())



from ..util.publisher import Publisher
import requests

class WebPublisher(Publisher):
    def __init__(self, web_url, web_secret, processor=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.web_url = web_url
        self.web_secret = web_secret
        self.processor = processor

    def __create_post_response(self, response):
        """
        Post the response to the configured URL.
        """
        response = self.parse_response(response)

        if self.processor:
            response = self.processor(response)
            
        body = {'payload': response, 'secret': self.web_secret}
        requests.post(self.web_url, json=body)



from setuptools import setup, find_packages

setup(
    name='cururo',
    version='1.1.3',
    author='Agustin Rios',
    author_email='arios6@uc.cl',
    description='Review an item using OpenAI.',
    long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.md').read(),
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=open("./requirements.txt").read().splitlines(),
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'cururo=cururo.cli:main',
        ],
    },
    classifiers=[
        # 'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        # 'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)



import functools
import logging

def handle_errors(default_value=None):
    """A decorator to handle exceptions in methods and optionally return a default value."""
    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as e:
                logging.error(f"An error occurred in {func.__name__}: {str(e)}")
                if default_value is not None:
                    return default_value
                raise  # Optionally re-raise the exception if no default value is provided
        return wrapper
    return decorator



import functools

class EnvModes:
    allowed_modes = ['production', 'development', 'testing']

    def __init__(self, mode='production') -> None:
        self.mode = mode
        self.validate_mode()

    def validate_mode(self):
        message = f"Mode must be one of {self.allowed_modes}"
        assert self.mode in self.allowed_modes, message

    def __call__(self, mode: str) -> None:
        self.mode = mode
        self.validate_mode()

    def __str__(self) -> str:
        return f"{self.__class__.__name__}(mode={self.mode})"
    
    def __repr__(self) -> str:
        return f"{self.__class__.__name__}(mode={self.mode})"
        
    def __enter__(self):
        return self
    
    @staticmethod
    def non_production_handler(return_value=None):
        """Decorator to handle non-production specific logic."""
        def decorator(func):
            @functools.wraps(func)
            def wrapper(self, *args, **kwargs):
                if self.mode != 'production':
                    return return_value
                return func(self, *args, **kwargs)
            return wrapper
        return decorator
        


import unittest
from .linter_test import LinterTest
from cururo.reviewer import Reviewer

class TestReviewer(LinterTest):
     def setUp(self):
         self.reviewer = Reviewer("api_key", "assistant_id")
         self.reviewer.assisant = None

if __name__ == '__main__':
    unittest.main()